!!$=head1 NAME
!!$
!!$Helicon - Helix convolution and deconvolution (N-dimensional!)
!!$
!!$=head1 SYNOPSIS
!!$
!!$Helicon < in.H filt= adj=0 div=0 > out.H
!!$
!!$=head1 DESCRIPTION
!!$
!!$Applies helix convolution (polynomial multiplication) or deconvolution
!!$(polynomial division). One is the exact inverse of the other. Watch
!!$for helical boundary conditions.
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history
!!$
!!$B<integer array> I<n> - reads I<n1>, I<n2>, I<n3>, ... 
!!$
!!$=item from par
!!$
!!$B<filename> I<filt> - helix filter file
!!$
!!$B<integer> I<adj>=0 - apply adjoint (backward) filtering
!!$
!!$B<integer> I<div>=0 - apply inverse recursive filtering (polynomial division)
!!$
!!$=item from aux (filt)
!!$
!!$B<integer array> I<h> - helix grid (can be I<h1>, I<h2>, ...)
!!$
!!$B<integer array> I<lag>=1,...,n1 - comma separated list of filter lags
!!$
!!$B<real> I<a0>=1 - zero-lag filter coefficient
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$F<gee/hlx/helderiv/>, F<gee/hlx/helicon90/>, F<gee/hlx/helocut90/>,
!!$F<gee/mda/morgan/>, F<gee/prc/sep95/>
!!$
!!$=head1 MODULES
!!$
!!$F<helicon.lop>, F<polydiv.lop>, F<regrid.r90>, F<helix.r90>
!!$
!!$=head1 SEA ALSO
!!$
!!$L<Missfill>, L<Pef>
!!$
!!$=cut
!%
program Helico
  use sep
  use helix
  use helicon
  use polydiv
  use regrid

  implicit none
  integer                             :: ia, na, nx, stat, dim
  real                                :: a0
  logical                             :: adj, add = .false., div
  real,    dimension (:), allocatable :: pp, qq
  integer, dimension (:), allocatable :: n, m
  type( filter)                       :: aa

  call sep_init (SOURCE)
  dim = sep_dimension()
  allocate (n (dim), m (dim))
  call from_history (n)
  call from_aux ("filt",na)
  call allocatehelix (aa, na)

  call from_aux ("filt", "a0", a0, 1.)
  call from_aux ("filt", "n", m)
  call from_aux ("filt", "lag", aa%lag, (/ (ia, ia = 1, na) /))
  call regridn (m, n, aa)

  call from_par ("adj",adj,.false.)
  call from_par ("div",div,.false.)
  call sep_close ()

  nx = product (n)
  allocate (pp (nx), qq (nx))
  if (adj) then
     call sep_read (qq)
  else
     call sep_read (pp)
  end if

  call sep_read (aa%flt, "filt"); aa%flt = aa%flt/a0
  if (div) then
     call polydiv_init (nx, aa)
     stat = polydiv_lop (adj,add,pp,qq)
     call polydiv_close()
  else
     call helicon_init (aa)
     stat = helicon_lop (adj,add,pp,qq)
  end if

  if (adj) then
     call sep_write (pp)
  else
     call sep_write (qq)
  end if

  deallocate (n, m, pp, qq)
  call deallocatehelix (aa)
  call exit (0)
end program Helico


